/**
  *  =====================================================================
  *  CarModel.java: Construct a 2D car model ...
  *  =====================================================================
  */

package visitor03.model.compound;

import visitor03.model.*;
import visitor03.model.primitive.*;

public class CarModel extends AbstractCompoundFeature implements LocationListener {
   public boolean DEBUG = true;
   public Point  body01, body02, body03, body04;
   public Point  roof01, roof02, roof03, roof04;
   public Edge   edge01, edge02, edge03, edge04, edge05, edge06, edge07;
   public Circle wheel01, wheel02;

   public CarModel( double x, double y, double width) {

      // Create label for the car model ...

      setName( "Car Model" );

      // Save corner reference point and width of shape ...

      setX( (double) x );
      setY( (double) y );
      setHeight( (double) (1.0/2.0)*width );
      setWidth( (double) width );

      // Initialize envelope ...
    
      be.addPoint( x, y );
      be.addPoint( x + width, y + width/2.0 );

      // Create vertices for car schematic ...

      body01 = new Point( "b01",         x,     y + width/6.0 );
      body02 = new Point( "b02",         x,     y + width/3.0 );
      body03 = new Point( "b03", x + width,     y + width/3.0 );
      body04 = new Point( "b04", x + width,     y + width/6.0 );

      // The bottom of the front windshield
      roof01 = new Point( "r01", x + width*5/6, y + width/3.0 );
      // The front of the roof
      roof02 = new Point( "r02", x + width*2/3, y + width/2.0 );
      // The rear of the roof
      roof03 = new Point( "r03",   x + width/3, y + width/2.0 );
      // The bottom of the rear windshield
      roof04 = new Point( "r04",   x + width/6, y + width/3.0 );

      // Connect edges to vertices ....

      edge01 = new Edge( "edge01",  body01, body02 );
      edge02 = new Edge( "edge02",  body02, body03 );
      edge03 = new Edge( "edge03",  body03, body04 );
      edge04 = new Edge( "edge04",  body04, body01 );
      edge05 = new Edge( "roof01",  roof01, roof02 );
      edge06 = new Edge( "roof02",  roof02, roof03 );
      edge07 = new Edge( "roof03",  roof03, roof04 );

      // Model the car wheels with circles ...

      wheel01 = new Circle( "wheel01",  x + width/3, width/12.0 );
      wheel01.setRadius( width/12.0 );
      wheel02 = new Circle( "wheel02",  x + width*2/3, width/12.0 );
      wheel02.setRadius( width/12.0 );

      // Add vertices to model ...

      items.put( body01.getName(), body01 );
      items.put( body02.getName(), body02 );
      items.put( body03.getName(), body03 );
      items.put( body04.getName(), body04 );

      items.put( roof01.getName(), roof01 );
      items.put( roof02.getName(), roof02 );
      items.put( roof03.getName(), roof03 );

      // Add edges to model ...

      items.put( edge01.getName(), edge01 );
      items.put( edge02.getName(), edge02 );
      items.put( edge03.getName(), edge03 );
      items.put( edge04.getName(), edge04 );
      items.put( edge05.getName(), edge05 );
      items.put( edge06.getName(), edge06 );
      items.put( edge07.getName(), edge07 );

      // Add wheels to model ...

      items.put( wheel01.getName(), wheel01 );
      items.put( wheel02.getName(), wheel02 );

      // Add this item to list of location listeners ....

      c.addLocationListener ( this );
   }

   // Implementation of location event handlers

   public void nodeMoved( LocationEvent evt ) {
      if ( DEBUG == true ) {
         System.out.println("*** In model.compound.CarModel.nodeMoved() ... ");
      }

      // Retrieve new position ....

      double x = getX();
      double y = getY();

      // Update nodal coordinates ...

      body01.setX(x);         body01.setY( y + width/6.0 );
      body02.setX(x);         body02.setY( y + width/3.0 );
      body03.setX(x + width); body03.setY( y + width/3.0 );
      body04.setX(x + width); body04.setY( y + width/6.0 );

      roof01.setX( x + width*5/6); roof01.setY(y + width/3.0 );
      roof02.setX( x + width*2/3); roof02.setY(y + width/2.0 );
      roof03.setX( x + width/3);   roof03.setY(y + width/2.0 );
      roof04.setX( x + width/6);   roof04.setY(y + width/3.0 );

      // Update (x,y) coordinates for the edges ...

      edge01.update( body01, body02 );
      edge02.update( body02, body03 );
      edge03.update( body03, body04 );
      edge04.update( body04, body01 );
      edge05.update( roof01, roof02 );
      edge06.update( roof02, roof03 );
      edge07.update( roof03, roof04 );

      wheel01.setX( x + width/3 );  wheel01.setY( width/12.0 );
      wheel02.setX( x + width*2/3); wheel02.setY( width/12.0 );

      // Update bounding envelope ...

      be.setToNull();
      be.addPoint( x, y );
      be.addPoint( x + width, y + width/2.0 );
   }

   // ==============================================================
   // Accept method for Feature Element visitor ...
   // ==============================================================

   public void accept( FeatureElementVisitor visitor) {
        visitor.visit(this);
   }

   // ==============================================================
   // Create string representation of car model ....
   // ==============================================================

   public String toString() {
      String s = "Car Model: Name = " + sName + "\n";
      s = s    + "     " + "Location (x,y) = (" + getX() + "," + getY() + ")";

      return s;
   }
}
